"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const rxjs_1 = require("@angular-devkit/schematics/node_modules/rxjs");
const index_1 = require("../utils/index");
function application(options) {
    const templateSource = schematics_1.apply(schematics_1.url('./files'), [
        schematics_1.template(Object.assign({}, options)),
        schematics_1.move(options.directory)
    ]);
    return schematics_1.chain([
        updateCLI(options),
        schematics_1.mergeWith(templateSource, schematics_1.MergeStrategy.Overwrite),
    ]);
}
exports.application = application;
function updateCLI(options) {
    return (tree, _context) => {
        return rxjs_1.Observable.create((subscriber) => {
            const defaultRule = schematics_1.externalSchematic('@schematics/angular', 'application', options);
            defaultRule(tree, _context).subscribe(tree => {
                const configSource = JSON.parse(index_1.getSource(tree, `${options.directory}/.angular-cli.json`));
                const devApp = configSource.apps.splice(0, 1)[0];
                const prodBrowser = JSON.parse(JSON.stringify(devApp));
                const prodServer = JSON.parse(JSON.stringify(devApp));
                configSource.apps.splice(0, 1);
                devApp.serviceWorker = true;
                devApp.outDir = `dist/browser`;
                devApp.assets.push({ glob: "**/*", input: "../firebug-lite", output: "./firebug-lite" });
                devApp.assets.push({ glob: "ngsw-worker.js", input: "./", output: "./ngsw-worker.js" });
                devApp.main = "main.browser.ts";
                devApp.styles = ["styles/main.scss"];
                devApp.environments = {
                    dev: "environments/environment.ts",
                    firebug: "environments/environment.firebug.ts",
                };
                prodBrowser.serviceWorker = true;
                prodBrowser.outDir = `dist/browser`;
                prodBrowser.assets.push('manifest.json');
                prodBrowser.main = "main.browser.ts";
                prodBrowser.styles = ["styles/main.scss"];
                prodBrowser.environments = {
                    prod: "environments/environment.prod.ts"
                };
                prodServer.outDir = `dist/server`;
                prodServer.assets.push('manifest.json');
                prodServer.main = "main.server.ts";
                prodServer.platform = "server";
                prodServer.tsconfig = "tsconfig.server.json";
                prodServer.styles = ["styles/main.scss"];
                prodServer.polyfills = null;
                prodServer.environments = {
                    prod: "environments/environment.prod.ts"
                };
                configSource.apps.push(devApp);
                configSource.apps.push(prodBrowser);
                configSource.apps.push(prodServer);
                tree.overwrite(`${options.directory}/.angular-cli.json`, JSON.stringify(configSource, null, "  "));
                subscriber.next(tree);
                subscriber.complete();
            });
        });
    };
}
//# sourceMappingURL=index.js.map